@@ -13,8 +13,6 @@
 #include <cstring>
 #include <cstdio>
 
-#include <sys/time.h>
-
 #include <Cocoa/Cocoa.h>
 
 #include "ScintillaTypes.h"
@@ -47,10 +45,6 @@ private:
 
 	CGContextRef gc;
 
-	/** The text layout instance */
-	std::unique_ptr<QuartzTextLayout> textLayout;
-	int verticalDeviceResolution;
-
 	/** If the surface is a bitmap context, contains a reference to the bitmap data. */
 	std::unique_ptr<uint8_t[]> bitmapData;
 	/** If the surface is a bitmap context, stores the dimensions of the bitmap. */
