@@ -6,7 +6,7 @@
 // Copyright 1998-2010 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <time.h>
+#include <ctime>
 
 #include <string>
 #include <chrono>
@@ -120,13 +120,18 @@ void Window::SetTitle(const char *s) {
 	gtk_window_set_title(GTK_WINDOW(wid), s);
 }
 
+void Window::SetRedraw(bool /* redraw */) {
+	// Could call gdk_window_freeze_updates / gdk_window_thaw_updates here
+	// but unsure what the side effects will be.
+}
+
 void Menu::CreatePopUp() {
 	Destroy();
 	mid = gtk_menu_new();
 	g_object_ref_sink(G_OBJECT(mid));
 }
 
-void Menu::Destroy() {
+void Menu::Destroy() noexcept {
 	if (mid)
 		g_object_unref(mid);
 	mid = 0;
