@@ -28,34 +28,36 @@
 
 using namespace Lexilla;
 
-static bool Is0To9(char ch) {
+namespace {
+
+constexpr bool Is0To9(char ch) noexcept {
 	return (ch >= '0') && (ch <= '9');
 }
 
-static bool IsAlphabetic(int ch) {
+bool IsAlphabetic(int ch) noexcept {
 	return IsASCII(ch) && isalpha(ch);
 }
 
-static inline bool AtEOL(Accessor &styler, Sci_PositionU i) {
+inline bool AtEOL(Accessor &styler, Sci_PositionU i) {
 	return (styler[i] == '\n') ||
 	       ((styler[i] == '\r') && (styler.SafeGetCharAt(i + 1) != '\n'));
 }
 
 // Tests for BATCH Operators
-static bool IsBOperator(char ch) {
+constexpr bool IsBOperator(char ch) noexcept {
 	return (ch == '=') || (ch == '+') || (ch == '>') || (ch == '<') ||
 		(ch == '|') || (ch == '?') || (ch == '*')||
 		(ch == '&') || (ch == '(') || (ch == ')');
 }
 
 // Tests for BATCH Separators
-static bool IsBSeparator(char ch) {
+constexpr bool IsBSeparator(char ch) noexcept {
 	return (ch == '\\') || (ch == '.') || (ch == ';') ||
 		(ch == '\"') || (ch == '\'') || (ch == '/');
 }
 
 // Tests for escape character
-static bool IsEscaped(char* wordStr, Sci_PositionU pos) {
+bool IsEscaped(const char* wordStr, Sci_PositionU pos) noexcept {
 	bool isQoted=false;
 	while (pos>0){
 		pos--;
@@ -68,7 +70,7 @@ static bool IsEscaped(char* wordStr, Sci_PositionU pos) {
 }
 
 // Tests for quote character
-static bool textQuoted(char *lineBuffer, Sci_PositionU endPos) {
+bool textQuoted(const char *lineBuffer, Sci_PositionU endPos) {
 	char strBuffer[1024];
 	strncpy(strBuffer, lineBuffer, endPos);
 	strBuffer[endPos] = '\0';
@@ -85,7 +87,7 @@ static bool textQuoted(char *lineBuffer, Sci_PositionU endPos) {
 	return CurrentStatus;
 }
 
-static void ColouriseBatchDoc(
+void ColouriseBatchDoc(
     Sci_PositionU startPos,
     Sci_Position length,
     int /*initStyle*/,
@@ -109,7 +111,7 @@ static void ColouriseBatchDoc(
 		}
 	}
 
-	char lineBuffer[1024];
+	char lineBuffer[1024] {};
 
 	styler.StartAt(startPos);
 	styler.StartSegment(startPos);
@@ -123,27 +125,18 @@ static void ColouriseBatchDoc(
 		if (AtEOL(styler, i) || (linePos >= sizeof(lineBuffer) - 1) || (i==startPos + length-1)) {
 			// End of line (or of line buffer) (or End of Last Line) met, colourise it
 			lineBuffer[linePos] = '\0';
-			Sci_PositionU lengthLine=linePos;
-			Sci_PositionU endPos=i;
-			Sci_PositionU offset = 0;	// Line Buffer Offset
-			Sci_PositionU cmdLoc;		// External Command / Program Location
-			char wordBuffer[81];		// Word Buffer - large to catch long paths
-			Sci_PositionU wbl;		// Word Buffer Length
-			Sci_PositionU wbo;		// Word Buffer Offset - also Special Keyword Buffer Length
-			WordList &keywords = *keywordlists[0];      // Internal Commands
-			WordList &keywords2 = *keywordlists[1];     // External Commands (optional)
+			const Sci_PositionU lengthLine=linePos;
+			const Sci_PositionU endPos=i;
+			const WordList &keywords = *keywordlists[0];      // Internal Commands
+			const WordList &keywords2 = *keywordlists[1];     // External Commands (optional)
 
 			// CHOICE, ECHO, GOTO, PROMPT and SET have Default Text that may contain Regular Keywords
 			//   Toggling Regular Keyword Checking off improves readability
 			// Other Regular Keywords and External Commands / Programs might also benefit from toggling
 			//   Need a more robust algorithm to properly toggle Regular Keyword Checking
 			bool stopLineProcessing=false;  // Used to stop line processing if Comment or Drive Change found
-			// Special Keywords are those that allow certain characters without whitespace after the command
-			// Examples are: cd. cd\ md. rd. dir| dir> echo: echo. path=
-			// Special Keyword Buffer used to determine if the first n characters is a Keyword
-			char sKeywordBuffer[10];	// Special Keyword Buffer
-			bool sKeywordFound;		// Exit Special Keyword for-loop if found
 
+			Sci_PositionU offset = 0;	// Line Buffer Offset
 			// Skip initial spaces
 			while ((offset < lengthLine) && (isspacechar(lineBuffer[offset]))) {
 				offset++;
@@ -151,7 +144,7 @@ static void ColouriseBatchDoc(
 			// Colorize Default Text
 			styler.ColourTo(startLine + offset - 1, SCE_BAT_DEFAULT);
 			// Set External Command / Program Location
-			cmdLoc = offset;
+			Sci_PositionU cmdLoc = offset;
 
 			// Check for Fake Label (Comment) or Real Label - return if found
 			if (lineBuffer[offset] == ':') {
@@ -190,19 +183,23 @@ static void ColouriseBatchDoc(
 					// Colorize Default Text
 					styler.ColourTo(startLine + offset - 1, SCE_BAT_DEFAULT);
 				}
+				char wordBuffer[81]{};		// Word Buffer - large to catch long paths
 				// Copy word from Line Buffer into Word Buffer
-				wbl = 0;
+				Sci_PositionU wbl = 0;		// Word Buffer Length
 				for (; offset < lengthLine && wbl < 80 &&
 						!isspacechar(lineBuffer[offset]); wbl++, offset++) {
-					wordBuffer[wbl] = static_cast<char>(tolower(lineBuffer[offset]));
+					wordBuffer[wbl] = tolower(lineBuffer[offset]);
 				}
 				wordBuffer[wbl] = '\0';
-				wbo = 0;
+				Sci_PositionU wbo = 0;		// Word Buffer Offset - also Special Keyword Buffer Length
 
 				// Check for Comment - return if found
-				if ((CompareCaseInsensitive(wordBuffer, "rem") == 0) && continueProcessing) {
-					styler.ColourTo(endPos, SCE_BAT_COMMENT);
-					break;
+				if (continueProcessing) {
+					if ((CompareCaseInsensitive(wordBuffer, "rem") == 0) || (wordBuffer[0] == ':' && wordBuffer[1] == ':')) {
+						styler.ColourTo(startLine + offset - strlen(wordBuffer) - 1, SCE_BAT_DEFAULT);
+						styler.ColourTo(endPos, SCE_BAT_COMMENT);
+						break;
+					}
 				}
 				// Check for Separator
 				if (IsBSeparator(wordBuffer[0])) {
@@ -287,12 +284,16 @@ static void ColouriseBatchDoc(
 					// Check for Special Keyword
 					//     Affected Commands are in Length range 2-6
 					//     Good that ERRORLEVEL, EXIST, CALL, DO, LOADHIGH, and LH are unaffected
-					sKeywordFound = false;
+					bool sKeywordFound = false;		// Exit Special Keyword for-loop if found
 					for (Sci_PositionU keywordLength = 2; keywordLength < wbl && keywordLength < 7 && !sKeywordFound; keywordLength++) {
+						// Special Keywords are those that allow certain characters without whitespace after the command
+						// Examples are: cd. cd\ md. rd. dir| dir> echo: echo. path=
+						// Special Keyword Buffer used to determine if the first n characters is a Keyword
+						char sKeywordBuffer[10]{};	// Special Keyword Buffer
 						wbo = 0;
 						// Copy Keyword Length from Word Buffer into Special Keyword Buffer
 						for (; wbo < keywordLength; wbo++) {
-							sKeywordBuffer[wbo] = static_cast<char>(wordBuffer[wbo]);
+							sKeywordBuffer[wbo] = wordBuffer[wbo];
 						}
 						sKeywordBuffer[wbo] = '\0';
 						// Check for Special Keyword in list
@@ -412,12 +413,12 @@ static void ColouriseBatchDoc(
 						if (cmdLoc == offset - wbl) {
 							cmdLoc = offset - (wbl - wbo);
 						}
-						bool isArgument = (wordBuffer[1] == '~');
+						const bool isArgument = (wordBuffer[1] == '~');
 						if (isArgument) {
 							Sci_PositionU expansionStopOffset = 2;
 							bool isValid = false;
 							for (; expansionStopOffset < wbl; expansionStopOffset++) {
-								if (isArgument && Is0To9(wordBuffer[expansionStopOffset])) {
+								if (Is0To9(wordBuffer[expansionStopOffset])) {
 									expansionStopOffset++;
 									isValid = true;
 									wbo = expansionStopOffset;
@@ -618,10 +619,12 @@ static void ColouriseBatchDoc(
 	}
 }
 
-static const char *const batchWordListDesc[] = {
+const char *const batchWordListDesc[] = {
 	"Internal Commands",
 	"External Commands",
 	0
 };
 
+}
+
 LexerModule lmBatch(SCLEX_BATCH, ColouriseBatchDoc, "batch", 0, batchWordListDesc);
