@@ -23,7 +23,7 @@ public:
 	PropSetSimple *pprops;
 	Accessor(Scintilla::IDocument *pAccess_, PropSetSimple *pprops_);
 	int GetPropertyInt(std::string_view key, int defaultValue=0) const;
-	int IndentAmount(Sci_Position line, int *flags, PFNIsCommentLeader pfnIsCommentLeader = 0);
+	int IndentAmount(Sci_Position line, int *flags, PFNIsCommentLeader pfnIsCommentLeader = nullptr);
 };
 
 }
