@@ -7,6 +7,7 @@
 
 #include <cstddef>
 #include <cstdlib>
+#include <cstdint>
 #include <cassert>
 #include <cstring>
 #include <cstdio>
@@ -86,7 +87,7 @@ void DrawWrapMarker(Surface *surface, PRectangle rcPlace,
 		XYPOSITION yBase;
 		int yDir;
 		XYPOSITION halfWidth;
-		Point At(XYPOSITION xRelative, XYPOSITION yRelative) noexcept {
+		Point At(XYPOSITION xRelative, XYPOSITION yRelative) const noexcept {
 			return Point(xBase + xDir * xRelative + halfWidth, yBase + yDir * yRelative + halfWidth);
 		}
 	};
