@@ -29,12 +29,9 @@ class PropSetFile {
 public:
 	PropSetFile *superPS;
 	explicit PropSetFile(bool lowerKeys_=false);
-	PropSetFile(const PropSetFile &copy);
-	PropSetFile &operator=(const PropSetFile &assign);
-	virtual ~PropSetFile();
 
 	void Set(std::string_view key, std::string_view val);
-	void SetLine(const char *keyVal);
+	void SetLine(const char *keyVal, bool unescape);
 	void Unset(std::string_view key);
 	bool Exists(const char *key) const;
 	std::string GetString(const char *key) const;
@@ -46,7 +43,7 @@ public:
 	long long GetLongLong(const char *key, long long defaultValue=0) const;
 	void Clear() noexcept;
 
-	enum ReadLineState { rlActive, rlExcludedModule, rlConditionFalse };
+	enum class ReadLineState { active, excludedModule, conditionFalse };
 	ReadLineState ReadLine(const char *lineBuffer, ReadLineState rls, const FilePath &directoryForImports, const ImportFilter &filter,
 			       FilePathSet *imports, size_t depth);
 	void ReadFromMemory(const char *data, size_t len, const FilePath &directoryForImports, const ImportFilter &filter,
