@@ -14,15 +14,7 @@ extern const GUI::gui_char propUserFileName[];
 extern const GUI::gui_char propGlobalFileName[];
 extern const GUI::gui_char propAbbrevFileName[];
 
-inline int Minimum(int a, int b) noexcept {
-	return (a < b) ? a : b;
-}
-
-inline int Maximum(int a, int b) noexcept {
-	return (a > b) ? a : b;
-}
-
-inline int IntFromTwoShorts(short a, short b) noexcept {
+constexpr int IntFromTwoShorts(short a, short b) noexcept {
 	return (a) | ((b) << 16);
 }
 
@@ -36,7 +28,7 @@ enum {
 	menuHelp = 8
 };
 
-namespace SA = Scintilla::API;
+namespace SA = Scintilla;
 
 constexpr int StyleMax = static_cast<int>(SA::StylesCommon::Max);
 constexpr int StyleDefault = static_cast<int>(SA::StylesCommon::Default);
@@ -59,11 +51,6 @@ public:
 	RecentFile(const FilePath &path_, SelectedRange selection_, SA::Line scrollPosition_) :
 		FilePath(path_), selection(selection_), scrollPosition(scrollPosition_) {
 	}
-	RecentFile(RecentFile const &) = default;
-	RecentFile(RecentFile &&) = default;
-	RecentFile &operator=(RecentFile const &) = default;
-	RecentFile &operator=(RecentFile &&) = default;
-	~RecentFile() override = default;
 	void Init() noexcept override {
 		FilePath::Init();
 		selection.position = SA::InvalidPosition;
@@ -87,50 +74,38 @@ public:
 
 struct FileWorker;
 
+// Scintilla documents can only be released by calling a method on a Scintilla
+// instance so store a Scintilla instance in the release functor
+struct BufferDocReleaser {
+	GUI::ScintillaWindow *pSci = nullptr;	// Non-owning
+	void operator()(void *pDoc) noexcept;
+};
+
+using BufferDoc = std::unique_ptr<void, BufferDocReleaser>;
+
 class Buffer {
 public:
 	RecentFile file;
-	void *doc;
+	BufferDoc doc;
 	bool isDirty;
 	bool isReadOnly;
 	bool failedSave;
 	bool useMonoFont;
-	enum { empty, reading, readAll, opened } lifeState;
+	enum class LifeState { empty, reading, readAll, opened } lifeState;
 	UniMode unicodeMode;
 	time_t fileModTime;
 	time_t fileModLastAsk;
 	time_t documentModTime;
-	enum { fmNone, fmTemporary, fmMarked, fmModified} findMarks;
+	enum class FindMarks { none, temporary, marked, modified} findMarks;
 	std::string overrideExtension;	///< User has chosen to use a particular language
 	std::vector<SA::Line> foldState;
 	std::vector<SA::Line> bookmarks;
-	FileWorker *pFileWorker;
+	std::unique_ptr<FileWorker> pFileWorker;
 	PropSetFile props;
-	enum FutureDo { fdNone=0, fdFinishSave=1 } futureDo;
-	Buffer() :
-		file(), doc(nullptr), isDirty(false), isReadOnly(false), failedSave(false), useMonoFont(false), lifeState(empty),
-		unicodeMode(uni8Bit), fileModTime(0), fileModLastAsk(0), documentModTime(0),
-		findMarks(fmNone), pFileWorker(nullptr), futureDo(fdNone) {}
+	enum class FutureDo { none=0, finishSave=1 } futureDo;
+	Buffer();
 
-	~Buffer() = default;
-	void Init() {
-		file.Init();
-		isDirty = false;
-		isReadOnly = false;
-		failedSave = false;
-		useMonoFont = false;
-		lifeState = empty;
-		unicodeMode = uni8Bit;
-		fileModTime = 0;
-		fileModLastAsk = 0;
-		documentModTime = 0;
-		findMarks = fmNone;
-		overrideExtension = "";
-		foldState.clear();
-		bookmarks.clear();
-		pFileWorker = nullptr;
-		futureDo = fdNone;
-	}
+	void Init();
 
 	void SetTimeFromFile() {
 		fileModTime = file.ModifiedTime();
@@ -147,12 +122,16 @@ public:
 	void AbandonAutomaticSave();
 
 	bool ShouldNotSave() const noexcept {
-		return lifeState != opened;
+		return lifeState != LifeState::opened;
 	}
 
 	void CancelLoad();
 };
 
+inline constexpr Buffer::FutureDo operator&(Buffer::FutureDo a, Buffer::FutureDo b) noexcept {
+	return static_cast<Buffer::FutureDo>(static_cast<int>(a) & static_cast<int>(b));
+}
+
 struct BackgroundActivities {
 	int loaders;
 	int storers;
@@ -173,7 +152,6 @@ public:
 	bool initialised;
 
 	BufferList();
-	~BufferList();
 	int size() const noexcept {
 		return static_cast<int>(buffers.size());
 	}
@@ -237,11 +215,11 @@ enum {
 };
 
 /// Codes representing the effect a line has on indentation.
-enum IndentationStatus {
-	isNone,		// no effect on indentation
-	isBlockStart,	// indentation block begin such as "{" or VB "function"
-	isBlockEnd,	// indentation end indicator such as "}" or VB "end"
-	isKeyWordStart	// Keywords that cause indentation
+enum class IndentationStatus {
+	none,		// no effect on indentation
+	blockStart,	// indentation block begin such as "{" or VB "function"
+	blockEnd,	// indentation end indicator such as "}" or VB "end"
+	keyWordStart	// Keywords that cause indentation
 };
 
 struct StyleAndWords {
@@ -254,7 +232,7 @@ struct StyleAndWords {
 };
 
 struct CurrentWordHighlight {
-	enum {
+	enum class StatesOfDelay {
 		noDelay,            // No delay, and no word at the caret.
 		delay,              // Delay before to highlight the word at the caret.
 		delayJustEnded,     // Delay has just ended. This state allows to ignore next HighlightCurrentWord (UpdateUI and SC_UPDATE_CONTENT for setting indicators).
@@ -266,7 +244,7 @@ struct CurrentWordHighlight {
 	bool isOnlyWithSameStyle;
 
 	CurrentWordHighlight() {
-		statesOfDelay = noDelay;
+		statesOfDelay = StatesOfDelay::noDelay;
 		isEnabled = false;
 		textHasChanged = false;
 		isOnlyWithSameStyle = false;
@@ -279,81 +257,18 @@ public:
 	bool read;
 	Localization() : PropSetFile(true), read(false) {
 	}
-	GUI::gui_string Text(const char *s, bool retainIfNotFound=true) override;
+	// Deleted so Localization objects can not be copied.
+	Localization(const Localization &) = delete;
+	Localization(Localization &&) = delete;
+	Localization &operator=(const Localization &) = delete;
+	Localization &operator=(Localization &&) = delete;
+	virtual ~Localization() = default;
+	GUI::gui_string Text(std::string_view sv, bool retainIfNotFound=true) const override;
 	void SetMissing(const std::string &missing_) {
 		missing = missing_;
 	}
 };
 
-// Interface between SciTE and dialogs and strips for find and replace
-class Searcher {
-public:
-	std::string findWhat;
-	std::string replaceWhat;
-
-	bool wholeWord;
-	bool matchCase;
-	bool regExp;
-	bool unSlash;
-	bool wrapFind;
-	bool reverseFind;
-
-	SA::Position searchStartPosition;
-	bool replacing;
-	bool havefound;
-	bool failedfind;
-	bool findInStyle;
-	int findStyle;
-	enum class CloseFind { closePrevent, closeAlways, closeOnMatch } closeFind;
-	ComboMemory memFinds;
-	ComboMemory memReplaces;
-
-	bool focusOnReplace;
-
-	Searcher();
-
-	virtual void SetFindText(const char *sFind) = 0;
-	virtual void SetFind(const char *sFind) = 0;
-	virtual bool FindHasText() const noexcept = 0;
-	void InsertFindInMemory();
-	virtual void SetReplace(const char *sReplace) = 0;
-	virtual void SetCaretAsStart() = 0;
-	virtual void MoveBack() = 0;
-	virtual void ScrollEditorIfNeeded() = 0;
-
-	virtual SA::Position FindNext(bool reverseDirection, bool showWarnings=true, bool allowRegExp=true) = 0;
-	virtual void HideMatch() = 0;
-	enum MarkPurpose { markWithBookMarks, markIncremental };
-	virtual void MarkAll(MarkPurpose purpose=markWithBookMarks) = 0;
-	virtual intptr_t ReplaceAll(bool inSelection) = 0;
-	virtual void ReplaceOnce(bool showWarnings=true) = 0;
-	virtual void UIClosed() = 0;
-	virtual void UIHasFocus() = 0;
-	bool &FlagFromCmd(int cmd) noexcept;
-	bool ShouldClose(bool found) const noexcept {
-		return (closeFind == CloseFind::closeAlways) || (found && (closeFind == CloseFind::closeOnMatch));
-	}
-};
-
-// User interface for search options implemented as both buttons and popup menu items
-struct SearchOption {
-	enum { tWord, tCase, tRegExp, tBackslash, tWrap, tUp };
-	const char *label;
-	int cmd;	// Menu item
-	int id;	// Control in dialog
-};
-
-class SearchUI {
-protected:
-	Searcher *pSearcher;
-public:
-	SearchUI() noexcept : pSearcher(nullptr) {
-	}
-	void SetSearcher(Searcher *pSearcher_) noexcept {
-		pSearcher = pSearcher_;
-	}
-};
-
 class IEditorConfig;
 struct SCNotification;
 
@@ -365,6 +280,9 @@ struct SystemAppearance {
 	}
 };
 
+// Titles appear different in menus and tabs
+enum class Title { menu, tab };
+
 class SciTEBase : public ExtensionAPI, public Searcher, public WorkerListener {
 protected:
 	bool needIdle;
@@ -382,12 +300,15 @@ protected:
 	enum { importCmdID = IDM_IMPORT };
 	ImportFilter filter;
 
-	enum { indicatorMatch = static_cast<int>(SA::IndicatorStyle::Container),
+	enum { indicatorMatch = static_cast<int>(SA::IndicatorNumbers::Container),
 	       indicatorHighlightCurrentWord,
 	       indicatorSpellingMistake,
 	       indicatorSentinel
 	     };
-	enum { markerBookmark = 1 };
+
+	static constexpr int markerBookmark = 1;
+	static constexpr int markerFilterMatch = 2;
+
 	ComboMemory memFiles;
 	ComboMemory memDirectory;
 	std::string parameterisedCommand;
@@ -403,11 +324,12 @@ protected:
 	SA::CharacterSet characterSet;
 	std::string language;
 	int lexLanguage;
+	std::vector<std::string> monospacedList;
 	std::string subStyleBases;
-	int lexLPeg;
 	StringList apis;
 	std::string apisFileNames;
 	std::string functionDefinition;
+	BufferDocReleaser docReleaser;
 
 	int diagnosticStyleStart;
 	enum { diagnosticStyles=4};
@@ -470,6 +392,8 @@ protected:
 	Extension *extender;
 	bool needReadProperties;
 	bool quitting;
+	bool canUndo;
+	bool canRedo;
 
 	int timerMask;
 	enum { timerAutoSave=1 };
@@ -495,6 +419,7 @@ protected:
 	bool callTipUseEscapes;
 	bool callTipIgnoreCase;
 	bool autoCCausedByOnlyOne;
+	int autoCompleteVisibleItemCount;
 	std::string calltipWordCharacters;
 	std::string calltipParametersStart;
 	std::string calltipParametersEnd;
@@ -560,6 +485,8 @@ protected:
 	// Handle buffers
 	void *GetDocumentAt(int index);
 	void SwitchDocumentAt(int index, void *pdoc);
+	void SaveFolds(std::vector<SA::Line> &folds);
+	void RestoreFolds(const std::vector<SA::Line> &folds);
 	void UpdateBuffersCurrent();
 	bool IsBufferAvailable() const noexcept;
 	bool CanMakeRoom(bool maySaveIfDirty = true);
@@ -578,6 +505,7 @@ protected:
 	void PrevInStack();
 	void EndStackedTabbing();
 
+	virtual void UpdateTabs(const std::vector<GUI::gui_string> &tabNames);
 	virtual void TabInsert(int index, const GUI::gui_char *title) = 0;
 	virtual void TabSelect(int index) = 0;
 	virtual void RemoveAllTabs() = 0;
@@ -654,7 +582,7 @@ protected:
 	void TextWritten(FileWorker *pFileWorker);
 	void UpdateProgress(Worker *pWorker);
 	void PerformDeferredTasks();
-	enum OpenCompletion { ocSynchronous, ocCompleteCurrent, ocCompleteSwitch };
+	enum class OpenCompletion { synchronous, completeCurrent, completeSwitch };
 	void CompleteOpen(OpenCompletion oc);
 	virtual bool PreOpenCheck(const GUI::gui_char *file);
 	bool Open(const FilePath &file, OpenFlags of = ofNone);
@@ -666,16 +594,16 @@ protected:
 		sfProgressVisible = 1, 	// Show in background save strip
 		sfSynchronous = 16	// Write synchronously blocking UI
 	};
-	enum SaveResult {
-		saveCompleted,
-		saveCancelled
+	enum class SaveResult {
+		completed,
+		cancelled
 	};
 	SaveResult SaveIfUnsure(bool forceQuestion = false, SaveFlags sf = sfProgressVisible);
 	SaveResult SaveIfUnsureAll();
 	SaveResult SaveIfUnsureForBuilt();
 	bool SaveIfNotOpen(const FilePath &destFile, bool fixCase);
 	void AbandonAutomaticSave();
-	bool Save(SaveFlags sf = sfProgressVisible);
+	virtual bool Save(SaveFlags sf = sfProgressVisible);
 	void SaveAs(const GUI::gui_char *file, bool fixCase);
 	virtual void SaveACopy() = 0;
 	void SaveToHTML(const FilePath &saveName);
@@ -702,7 +630,7 @@ protected:
 	FilePath GetLocalPropertiesFileName();
 	FilePath GetAbbrevPropertiesFileName();
 	void OpenProperties(int propsFile);
-	static int GetMenuCommandAsInt(std::string commandName);
+	static int GetMenuCommandAsInt(const std::string &commandName);
 	virtual void Print(bool) {}
 	virtual void PrintSetup() {}
 	void UserStripShow(const char * /* description */) override {}
@@ -710,30 +638,30 @@ protected:
 	void UserStripSetList(int /* control */, const char * /* value */) override {}
 	std::string UserStripValue(int /* control */) override { return std::string(); }
 	virtual void ShowBackgroundProgress(const GUI::gui_string & /* explanation */, size_t /* size */, size_t /* progress */) {}
-	SA::Range GetSelection();
+	SA::Span GetSelection();
 	SelectedRange GetSelectedRange();
 	void SetSelection(SA::Position anchor, SA::Position currentPos);
 	std::string GetCTag();
-	virtual std::string GetRangeInUIEncoding(GUI::ScintillaWindow &win, SA::Range range);
+	virtual std::string GetRangeInUIEncoding(GUI::ScintillaWindow &win, SA::Span span);
 	static std::string GetLine(GUI::ScintillaWindow &win, SA::Line line);
-	void RangeExtend(GUI::ScintillaWindow &wCurrent, SA::Range &range,
+	void RangeExtend(GUI::ScintillaWindow &wCurrent, SA::Span &range,
 			 bool (SciTEBase::*ischarforsel)(char ch));
-	std::string RangeExtendAndGrab(GUI::ScintillaWindow &wCurrent, SA::Range &range,
+	std::string RangeExtendAndGrab(GUI::ScintillaWindow &wCurrent, SA::Span &span,
 				       bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
 	std::string SelectionExtend(bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
 	std::string SelectionWord(bool stripEol = true);
 	std::string SelectionFilename();
 	void SelectionIntoProperties();
 	void SelectionIntoFind(bool stripEol = true);
-	enum AddSelection { addNext, addEach };
+	enum class AddSelection { next, each };
 	void SelectionAdd(AddSelection add);
 	virtual std::string EncodeString(const std::string &s);
 	virtual void Find() = 0;
-	enum MessageBoxChoice {
-		mbOK,
-		mbCancel,
-		mbYes,
-		mbNo
+	enum class MessageBoxChoice {
+		ok,
+		cancel,
+		yes,
+		no
 	};
 	typedef int MessageBoxStyle;
 	enum {
@@ -748,12 +676,12 @@ protected:
 	void FailedSaveMessageBox(const FilePath &filePathSaving);
 	virtual void FindMessageBox(const std::string &msg, const std::string *findItem = nullptr) = 0;
 	bool FindReplaceAdvanced() const;
-	SA::Position FindInTarget(const std::string &findWhatText, SA::Range range);
+	SA::Position FindInTarget(const std::string &findWhatText, SA::Span range);
 	// Implement Searcher
-	void SetFindText(const char *sFind) override;
-	void SetFind(const char *sFind) override;
+	void SetFindText(std::string_view sFind) override;
+	void SetFind(std::string_view sFind) override;
 	bool FindHasText() const noexcept override;
-	void SetReplace(const char *sReplace) override;
+	void SetReplace(std::string_view sReplace) override;
 	void SetCaretAsStart() override;
 	void MoveBack() override;
 	void ScrollEditorIfNeeded() override;
@@ -761,6 +689,9 @@ protected:
 	void HideMatch() override;
 	virtual void FindIncrement() = 0;
 	int IncrementSearchMode();
+	virtual void Filter() = 0;
+	virtual bool FilterShowing() { return false; }
+	int FilterSearch();
 	virtual void FindInFiles() = 0;
 	virtual void Replace() = 0;
 	void ReplaceOnce(bool showWarnings=true) override;
@@ -772,7 +703,7 @@ protected:
 	void UIHasFocus() override;
 	virtual void DestroyFindReplace() = 0;
 	virtual void GoLineDialog() = 0;
-	virtual bool AbbrevDialog() = 0;
+	virtual void AbbrevDialog() = 0;
 	virtual void TabSizeDialog() = 0;
 	virtual bool ParametersOpen() = 0;
 	virtual void ParamGrab() = 0;
@@ -798,7 +729,6 @@ protected:
 	virtual bool StartAutoCompleteWord(bool onlyOneWord);
 	virtual bool StartExpandAbbreviation();
 	bool PerformInsertAbbreviation();
-	virtual bool StartInsertAbbreviation();
 	virtual bool StartBlockComment();
 	virtual bool StartBoxComment();
 	virtual bool StartStreamComment();
@@ -832,6 +762,7 @@ protected:
 	void SaveTitledBuffers();
 	virtual void CopyAsRTF() {}
 	virtual void CopyPath() {}
+	void SetFoldWidth();
 	void SetLineNumberWidth();
 	void MenuCommand(int cmdID, int source = 0);
 	void FoldChanged(SA::Line line, SA::FoldLevel levelNow, SA::FoldLevel levelPrev);
@@ -839,12 +770,14 @@ protected:
 	void FoldAll();
 	void ToggleFoldRecursive(SA::Line line, SA::FoldLevel level);
 	void EnsureAllChildrenVisible(SA::Line line, SA::FoldLevel level);
-	static void EnsureRangeVisible(GUI::ScintillaWindow &win, SA::Range range, bool enforcePolicy = true);
+	static void EnsureRangeVisible(GUI::ScintillaWindow &win, SA::Span range, bool enforcePolicy = true);
 	void GotoLineEnsureVisible(SA::Line line);
 	bool MarginClick(SA::Position position, int modifiers);
 	void NewLineInOutput();
 	virtual void SetStatusBarText(const char *s) = 0;
 	void UpdateUI(const SCNotification *notification);
+	void SetCanUndoRedo(bool canUndo_, bool canRedo_);
+	void CheckCanUndoRedo();
 	void Modified(const SCNotification *notification);
 	virtual void Notify(SCNotification *notification);
 	virtual void ShowToolBar() = 0;
@@ -854,7 +787,8 @@ protected:
 
 	void RemoveFindMarks();
 	SA::FindOption SearchFlags(bool regularExpressions) const;
-	void MarkAll(MarkPurpose purpose=markWithBookMarks) override;
+	void MarkAll(MarkPurpose purpose) override;
+	void FilterAll(bool showMatches) override;
 	void BookmarkAdd(SA::Line lineno = -1);
 	void BookmarkDelete(SA::Line lineno = -1);
 	bool BookmarkPresent(SA::Line lineno = -1);
@@ -913,12 +847,21 @@ protected:
 	void SetOverrideLanguage(int cmdID);
 	StyleAndWords GetStyleAndWords(const char *base);
 	std::string ExtensionFileName() const;
+	void SetElementColour(SA::Element element, const char *key);
 	static const char *GetNextPropItem(const char *pStart, char *pPropItem, int maxLen);
 	void ForwardPropertyToEditor(const char *key);
-	void DefineMarker(SA::MarkerOutline marker, SA::MarkerSymbol markerType, SA::Colour fore, SA::Colour back, SA::Colour backSelected);
+	struct MarkerAppearance {
+		SA::ColourAlpha fore;
+		SA::ColourAlpha back;
+		SA::ColourAlpha backSelected;
+		int strokeWidth;
+	};
+	void DefineMarker(SA::MarkerOutline marker, SA::MarkerSymbol markerType, MarkerAppearance markerAppearance);
 	void ReadAPI(const std::string &fileNameForExtension);
 	std::string FindLanguageProperty(const char *pattern, const char *defaultValue = "");
+	void SetRepresentations();
 	virtual void ReadProperties();
+	void ReadEditorConfig(const std::string &fileNameForExtension);
 	std::string StyleString(const char *lang, int style) const;
 	StyleDefinition StyleDefinitionFor(int style);
 	void SetOneStyle(GUI::ScintillaWindow &win, int style, const StyleDefinition &sd);
@@ -969,7 +912,7 @@ protected:
 	void PropertyToDirector(const char *arg);
 	// ExtensionAPI
 	intptr_t Send(Pane p, SA::Message msg, uintptr_t wParam = 0, intptr_t lParam = 0) override;
-	std::string Range(Pane p, SA::Range range) override;
+	std::string Range(Pane p, SA::Span range) override;
 	void Remove(Pane p, SA::Position start, SA::Position end) override;
 	void Insert(Pane p, SA::Position pos, const char *s) override;
 	void Trace(const char *s) override;
@@ -1015,6 +958,7 @@ public:
 
 int ControlIDOfCommand(unsigned long) noexcept;
 SA::Colour ColourOfProperty(const PropSetFile &props, const char *key, SA::Colour colourDefault);
+SA::ColourAlpha ColourAlphaOfProperty(const PropSetFile &props, const char *key, SA::ColourAlpha colourDefault);
 void WindowSetFocus(GUI::ScintillaWindow &w);
 
 // Test if an enum class value has the bit flag(s) of test set.
