@@ -13,11 +13,13 @@
 #include <cstdio>
 #include <ctime>
 
+#include <tuple>
 #include <string>
 #include <string_view>
 #include <vector>
 #include <map>
 #include <set>
+#include <optional>
 #include <algorithm>
 #include <memory>
 #include <chrono>
@@ -47,10 +49,46 @@
 #include "Worker.h"
 #include "FileWorker.h"
 #include "MatchMarker.h"
+#include "Searcher.h"
 #include "SciTEBase.h"
 
 const GUI::gui_char defaultSessionFileName[] = GUI_TEXT("SciTE.session");
 
+void BufferDocReleaser::operator()(void *pDoc) noexcept {
+	if (pDoc) {
+		try {
+			pSci->ReleaseDocument(pDoc);
+		} catch (...) {
+			// ReleaseDocument must not throw, ignore if it does.
+		}
+	}
+}
+
+Buffer::Buffer() :
+	file(), isDirty(false), isReadOnly(false), failedSave(false), useMonoFont(false), lifeState(LifeState::empty),
+	unicodeMode(UniMode::uni8Bit), fileModTime(0), fileModLastAsk(0), documentModTime(0),
+	findMarks(FindMarks::none), futureDo(FutureDo::none) {}
+
+void Buffer::Init() {
+	file.Init();
+	isDirty = false;
+	isReadOnly = false;
+	failedSave = false;
+	useMonoFont = false;
+	lifeState = LifeState::empty;
+	unicodeMode = UniMode::uni8Bit;
+	fileModTime = 0;
+	fileModLastAsk = 0;
+	documentModTime = 0;
+	findMarks = FindMarks::none;
+	overrideExtension = "";
+	foldState.clear();
+	bookmarks.clear();
+	pFileWorker.reset();
+	futureDo = FutureDo::none;
+	doc.reset();
+}
+
 void Buffer::DocumentModified() noexcept {
 	documentModTime = time(nullptr);
 }
@@ -61,25 +99,23 @@ bool Buffer::NeedsSave(int delayBeforeSave) const {
 }
 
 void Buffer::CompleteLoading() noexcept {
-	lifeState = opened;
+	lifeState = LifeState::opened;
 	if (pFileWorker && pFileWorker->IsLoading()) {
-		delete pFileWorker;
-		pFileWorker = nullptr;
+		pFileWorker.reset();
 	}
 }
 
 void Buffer::CompleteStoring() {
 	if (pFileWorker && !pFileWorker->IsLoading()) {
-		delete pFileWorker;
-		pFileWorker = nullptr;
+		pFileWorker.reset();
 	}
 	SetTimeFromFile();
 }
 
 void Buffer::AbandonAutomaticSave() {
 	if (pFileWorker && !pFileWorker->IsLoading()) {
-		const FileStorer *pFileStorer = static_cast<FileStorer *>(pFileWorker);
-		if (!pFileStorer->visibleProgress) {
+		const FileStorer *pFileStorer = dynamic_cast<FileStorer *>(pFileWorker.get());
+		if (pFileStorer && !pFileStorer->visibleProgress) {
 			pFileWorker->Cancel();
 			// File is in partially saved state so may be better to remove
 		}
@@ -91,15 +127,12 @@ void Buffer::CancelLoad() {
 	if (pFileWorker && pFileWorker->IsLoading()) {
 		pFileWorker->Cancel();
 		CompleteLoading();
-		lifeState = empty;
+		lifeState = LifeState::empty;
 	}
 }
 
 BufferList::BufferList() : current(0), stackcurrent(0), stack(0), buffers(0), length(0), lengthVisible(0), initialised(false) {}
 
-BufferList::~BufferList() {
-}
-
 void BufferList::Allocate(int maxSize) {
 	length = 1;
 	lengthVisible = 1;
@@ -123,7 +156,7 @@ int BufferList::Add() {
 
 int BufferList::GetDocumentByWorker(const FileWorker *pFileWorker) const {
 	for (int i = 0; i < length; i++) {
-		if (buffers[i].pFileWorker == pFileWorker) {
+		if (buffers[i].pFileWorker.get() == pFileWorker) {
 			return i;
 		}
 	}
@@ -156,13 +189,11 @@ void BufferList::RemoveInvisible(int index) {
 }
 
 void BufferList::RemoveCurrent() {
-	// Delete and move up to fill gap but ensure doc pointer is saved.
-	void *currentDoc = buffers[current].doc;
+	// Delete and move up to fill gap.
 	buffers[current].CompleteLoading();
 	for (int i = current; i < length - 1; i++) {
-		buffers[i] = buffers[i + 1];
+		buffers[i] = std::move(buffers[i + 1]);
 	}
-	buffers[length - 1].doc = currentDoc;
 
 	if (length > 1) {
 		CommitStackSelection();
@@ -247,12 +278,12 @@ void BufferList::ShiftTo(int indexFrom, int indexTo) {
 			indexFrom < 0 || indexFrom >= length ||
 			indexTo < 0 || indexTo >= length) return;
 	int step = (indexFrom > indexTo) ? -1 : 1;
-	Buffer tmp = buffers[indexFrom];
+	Buffer tmp = std::move(buffers[indexFrom]);
 	int i;
 	for (i = indexFrom; i != indexTo; i += step) {
-		buffers[i] = buffers[i+step];
+		buffers[i] = std::move(buffers[i+step]);
 	}
-	buffers[indexTo] = tmp;
+	buffers[indexTo] = std::move(tmp);
 	// update stack indexes
 	for (i = 0; i < length; i++) {
 		if (stack[i] == indexFrom) {
@@ -270,9 +301,7 @@ void BufferList::Swap(int indexA, int indexB) {
 	if (indexA == indexB ||
 			indexA < 0 || indexA >= length ||
 			indexB < 0 || indexB >= length) return;
-	Buffer tmp = buffers[indexA];
-	buffers[indexA] = buffers[indexB];
-	buffers[indexB] = tmp;
+	std::swap(buffers[indexA], buffers[indexB]);
 	// update stack indexes
 	for (int i = 0; i < length; i++) {
 		if (stack[i] == indexA) {
@@ -293,8 +322,8 @@ BackgroundActivities BufferList::CountBackgroundActivities() const {
 		if (buffers[i].pFileWorker) {
 			if (!buffers[i].pFileWorker->FinishedJob()) {
 				if (!buffers[i].pFileWorker->IsLoading()) {
-					const FileStorer *fstorer = static_cast<FileStorer *>(buffers[i].pFileWorker);
-					if (!fstorer->visibleProgress)
+					const FileStorer *fstorer = dynamic_cast<FileStorer *>(buffers[i].pFileWorker.get());
+					if (fstorer && !fstorer->visibleProgress)
 						continue;
 				}
 				if (buffers[i].pFileWorker->IsLoading())
@@ -343,6 +372,14 @@ void BufferList::SetVisible(int index, bool visible) {
 	}
 }
 
+constexpr Buffer::FutureDo operator|(Buffer::FutureDo a, Buffer::FutureDo b) noexcept {
+	return static_cast<Buffer::FutureDo>(static_cast<int>(a) | static_cast<int>(b));
+}
+
+constexpr Buffer::FutureDo operator~(Buffer::FutureDo a) noexcept {
+	return static_cast<Buffer::FutureDo>(~static_cast<int>(a));
+}
+
 void BufferList::AddFuture(int index, Buffer::FutureDo fd) {
 	if (index >= 0 || index < length) {
 		buffers[index].futureDo = static_cast<Buffer::FutureDo>(buffers[index].futureDo | fd);
@@ -359,24 +396,20 @@ void *SciTEBase::GetDocumentAt(int index) {
 	if (index < 0 || index >= buffers.size()) {
 		return nullptr;
 	}
-	if (buffers.buffers[index].doc == nullptr) {
+	if (!buffers.buffers[index].doc) {
 		// Create a new document buffer
-		buffers.buffers[index].doc = wEditor.CreateDocument(0, SA::DocumentOption::Default);
+		buffers.buffers[index].doc = BufferDoc(wEditor.CreateDocument(0, SA::DocumentOption::Default), docReleaser);
 	}
-	return buffers.buffers[index].doc;
+	return buffers.buffers[index].doc.get();
 }
 
 void SciTEBase::SwitchDocumentAt(int index, void *pdoc) {
 	if (index < 0 || index >= buffers.size()) {
 		return;
 	}
-	void *pdocOld = buffers.buffers[index].doc;
-	buffers.buffers[index].doc = pdoc;
-	if (pdocOld) {
-		wEditor.ReleaseDocument(pdocOld);
-	}
+	buffers.buffers[index].doc = BufferDoc(pdoc, docReleaser);
 	if (index == buffers.Current()) {
-		wEditor.SetDocPointer(buffers.buffers[index].doc);
+		wEditor.SetDocPointer(buffers.buffers[index].doc.get());
 	}
 }
 
@@ -409,10 +442,10 @@ void SciTEBase::SetDocumentAt(int index, bool updateStack) {
 	propsDiscovered = bufferNext.props;
 	propsDiscovered.superPS = &propsLocal;
 	wEditor.SetDocPointer(GetDocumentAt(buffers.Current()));
-	const bool restoreBookmarks = bufferNext.lifeState == Buffer::readAll;
+	const bool restoreBookmarks = bufferNext.lifeState == Buffer::LifeState::readAll;
 	PerformDeferredTasks();
-	if (bufferNext.lifeState == Buffer::readAll) {
-		CompleteOpen(ocCompleteSwitch);
+	if (bufferNext.lifeState == Buffer::LifeState::readAll) {
+		CompleteOpen(OpenCompletion::completeSwitch);
 		if (extender)
 			extender->OnOpen(filePath.AsUTF8().c_str());
 	}
@@ -439,30 +472,38 @@ void SciTEBase::SetDocumentAt(int index, bool updateStack) {
 	}
 }
 
+void SciTEBase::SaveFolds(std::vector<SA::Line> &folds) {
+	folds.clear();
+
+	for (SA::Line line = 0; ; line++) {
+		const SA::Line lineNext = wEditor.ContractedFoldNext(line);
+		if ((line < 0) || (lineNext < line))
+			break;
+		line = lineNext;
+		folds.push_back(line);
+	}
+}
+void SciTEBase::RestoreFolds(const std::vector<SA::Line> &folds) {
+	for (const SA::Line fold : folds) {
+		wEditor.ToggleFold(fold);
+	}
+}
+
 void SciTEBase::UpdateBuffersCurrent() {
 	const int currentbuf = buffers.Current();
 
 	if ((buffers.length > 0) && (currentbuf >= 0) && (buffers.GetVisible(currentbuf))) {
 		Buffer &bufferCurrent = buffers.buffers[currentbuf];
 		bufferCurrent.file.Set(filePath);
-		if (bufferCurrent.lifeState != Buffer::reading && bufferCurrent.lifeState != Buffer::readAll) {
+		if (bufferCurrent.lifeState != Buffer::LifeState::reading && bufferCurrent.lifeState != Buffer::LifeState::readAll) {
 			bufferCurrent.file.selection.position = wEditor.CurrentPos();
 			bufferCurrent.file.selection.anchor = wEditor.Anchor();
 			bufferCurrent.file.scrollPosition = GetCurrentScrollPosition();
 
 			// Retrieve fold state and store in buffer state info
 
-			std::vector<SA::Line> *f = &bufferCurrent.foldState;
-			f->clear();
-
-			if (props.GetInt("fold")) {
-				for (SA::Line line = 0; ; line++) {
-					const SA::Line lineNext = wEditor.ContractedFoldNext(line);
-					if ((line < 0) || (lineNext < line))
-						break;
-					line = lineNext;
-					f->push_back(line);
-				}
+			if (!FilterShowing() && props.GetInt("fold")) {
+				SaveFolds(bufferCurrent.foldState);
 			}
 
 			if (props.GetInt("session.bookmarks")) {
@@ -485,7 +526,7 @@ bool SciTEBase::CanMakeRoom(bool maySaveIfDirty) {
 		return true;
 	} else if (maySaveIfDirty) {
 		// All available buffers are taken, try and close the current one
-		if (SaveIfUnsure(true, static_cast<SaveFlags>(sfProgressVisible | sfSynchronous)) != saveCancelled) {
+		if (SaveIfUnsure(true, static_cast<SaveFlags>(sfProgressVisible | sfSynchronous)) != SaveResult::cancelled) {
 			// The file isn't dirty, or the user agreed to close the current one
 			return true;
 		}
@@ -520,8 +561,8 @@ void SciTEBase::InitialiseBuffers() {
 	if (!buffers.initialised) {
 		buffers.initialised = true;
 		// First document is the default from creation of control
-		buffers.buffers[0].doc = wEditor.DocPointer();
-		wEditor.AddRefDocument(buffers.buffers[0].doc); // We own this reference
+		buffers.buffers[0].doc = BufferDoc(wEditor.DocPointer(), docReleaser);
+		wEditor.AddRefDocument(buffers.buffers[0].doc.get()); // We own this reference
 		if (buffers.size() == 1) {
 			// Single buffer mode, delete the Buffers main menu entry
 			DestroyMenuItem(menuBuffers, 0);
@@ -625,19 +666,19 @@ void SciTEBase::RestoreFromSession(const Session &session) {
 void SciTEBase::RestoreSession() {
 	if (props.GetInt("save.find") != 0) {
 		for (int i = 0;; i++) {
-			std::string propKey = IndexPropKey("search", i, "findwhat");
-			std::string propStr = propsSession.GetString(propKey.c_str());
+			const std::string propKey = IndexPropKey("search", i, "findwhat");
+			const std::string propStr = propsSession.GetString(propKey.c_str());
 			if (propStr == "")
 				break;
-			memFinds.AppendList(propStr.c_str());
+			memFinds.Append(propStr);
 		}
 
 		for (int i = 0;; i++) {
-			std::string propKey = IndexPropKey("search", i, "replacewith");
-			std::string propStr = propsSession.GetString(propKey.c_str());
+			const std::string propKey = IndexPropKey("search", i, "replacewith");
+			const std::string propStr = propsSession.GetString(propKey.c_str());
 			if (propStr == "")
 				break;
-			memReplaces.AppendList(propStr.c_str());
+			memReplaces.Append(propStr);
 		}
 	}
 
@@ -746,7 +787,6 @@ void SciTEBase::SaveSessionFile(const GUI::gui_char *sessionName) {
 		mem = memReplaces.AsVector();
 		if (!mem.empty()) {
 			fprintf(sessionFile, "\n");
-			mem = memReplaces.AsVector();
 			it = mem.begin();
 			for (int i = 0; it != mem.end(); i++, ++it) {
 				propKey = IndexPropKey("search", i, "replacewith");
@@ -879,7 +919,7 @@ void SciTEBase::New() {
 	SetBuffersMenu();
 	CurrentBuffer()->isDirty = false;
 	CurrentBuffer()->failedSave = false;
-	CurrentBuffer()->lifeState = Buffer::opened;
+	CurrentBuffer()->lifeState = Buffer::LifeState::opened;
 	jobQueue.isBuilding = false;
 	jobQueue.isBuilt = false;
 	CurrentBuffer()->isReadOnly = false;	// No sense to create an empty, read-only buffer...
@@ -894,24 +934,26 @@ void SciTEBase::New() {
 void SciTEBase::RestoreState(const Buffer &buffer, bool restoreBookmarks) {
 	SetWindowName();
 	ReadProperties();
-	if (CurrentBuffer()->unicodeMode != uni8Bit) {
+	if (CurrentBuffer()->unicodeMode != UniMode::uni8Bit) {
 		// Override the code page if Unicode
 		codePage = SA::CpUtf8;
 		wEditor.SetCodePage(codePage);
 	}
 
+	RemoveFindMarks();
 	// check to see whether there is saved fold state, restore
-	if (!buffer.foldState.empty()) {
+	if (!buffer.foldState.empty() && !FilterShowing()) {
 		wEditor.ColouriseAll();
-		for (const SA::Line fold : buffer.foldState) {
-			wEditor.ToggleFold(fold);
-		}
+		RestoreFolds(buffer.foldState);
 	}
 	if (restoreBookmarks) {
 		for (const SA::Line bookmark : buffer.bookmarks) {
 			wEditor.MarkerAdd(bookmark, markerBookmark);
 		}
 	}
+	if (FilterShowing()) {
+		FilterAll(true);
+	}
 }
 
 void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew) {
@@ -941,7 +983,7 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 		closingLast = (buffers.lengthVisible == 1) && !buffers.buffers[0].pFileWorker;
 		if (closingLast) {
 			buffers.buffers[0].Init();
-			buffers.buffers[0].lifeState = Buffer::opened;
+			buffers.buffers[0].lifeState = Buffer::LifeState::opened;
 			if (extender)
 				extender->InitBuffer(0);
 		} else {
@@ -969,9 +1011,9 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 		propsDiscovered.superPS = &propsLocal;
 		wEditor.SetDocPointer(GetDocumentAt(buffers.Current()));
 		PerformDeferredTasks();
-		if (bufferNext.lifeState == Buffer::readAll) {
+		if (bufferNext.lifeState == Buffer::LifeState::readAll) {
 			//restoreBookmarks = true;
-			CompleteOpen(ocCompleteSwitch);
+			CompleteOpen(OpenCompletion::completeSwitch);
 			if (extender)
 				extender->OnOpen(filePath.AsUTF8().c_str());
 		}
@@ -1003,14 +1045,14 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 void SciTEBase::CloseTab(int tab) {
 	const int tabCurrent = buffers.Current();
 	if (tab == tabCurrent) {
-		if (SaveIfUnsure() != saveCancelled) {
+		if (SaveIfUnsure() != SaveResult::cancelled) {
 			Close();
 			WindowSetFocus(wEditor);
 		}
 	} else {
 		FilePath fpCurrent = buffers.buffers[tabCurrent].file.AbsolutePath();
 		SetDocumentAt(tab);
-		if (SaveIfUnsure() != saveCancelled) {
+		if (SaveIfUnsure() != SaveResult::cancelled) {
 			Close();
 			WindowSetFocus(wEditor);
 			// Return to the previous buffer
@@ -1020,7 +1062,7 @@ void SciTEBase::CloseTab(int tab) {
 }
 
 void SciTEBase::CloseAllBuffers(bool loadingSession) {
-	if (SaveAllBuffers(false) != saveCancelled) {
+	if (SaveAllBuffers(false) != SaveResult::cancelled) {
 		while (buffers.lengthVisible > 1)
 			Close(false, loadingSession);
 
@@ -1029,15 +1071,15 @@ void SciTEBase::CloseAllBuffers(bool loadingSession) {
 }
 
 SciTEBase::SaveResult SciTEBase::SaveAllBuffers(bool alwaysYes) {
-	SaveResult choice = saveCompleted;
+	SaveResult choice = SaveResult::completed;
 	UpdateBuffersCurrent();
 	const int currentBuffer = buffers.Current();
-	for (int i = 0; (i < buffers.lengthVisible) && (choice != saveCancelled); i++) {
+	for (int i = 0; (i < buffers.lengthVisible) && (choice != SaveResult::cancelled); i++) {
 		if (buffers.buffers[i].isDirty) {
 			SetDocumentAt(i);
 			if (alwaysYes) {
 				if (!Save()) {
-					choice = saveCancelled;
+					choice = SaveResult::cancelled;
 				}
 			} else {
 				choice = SaveIfUnsure(false);
@@ -1117,23 +1159,98 @@ void SciTEBase::EndStackedTabbing() {
 	buffers.CommitStackSelection();
 }
 
-static void EscapeFilePathsForMenu(GUI::gui_string &path) {
-	// Escape '&' characters in path, since they are interpreted in
-	// menus.
-	Substitute(path, GUI_TEXT("&"), GUI_TEXT("&&"));
-#if defined(GTK)
-	GUI::gui_string homeDirectory = getenv("HOME");
-	if (StartsWith(path, homeDirectory)) {
-		path.replace(static_cast<size_t>(0), homeDirectory.size(), GUI_TEXT("~"));
+void SciTEBase::UpdateTabs(const std::vector<GUI::gui_string> &tabNames) {
+	RemoveAllTabs();
+	for (int t = 0; t < static_cast<int>(tabNames.size()); t++) {
+		TabInsert(t, tabNames[t].c_str());
+	}
+}
+
+namespace {
+
+GUI::gui_string EscapeFilePath(const FilePath &path, [[maybe_unused]]Title destination) {
+	// Escape '&' characters in path, since they are interpreted in menus.
+	GUI::gui_string escaped(path.AsInternal());
+#if defined(_WIN32)
+	// On Windows, '&' are interpreted in menus and tab names, so we need
+	// the escaped filename
+	Substitute(escaped, GUI_TEXT("&"), GUI_TEXT("&&"));
+#else
+	if (destination == Title::menu) {
+		Substitute(escaped, GUI_TEXT("&"), GUI_TEXT("&&"));
 	}
 #endif
+	return escaped;
+}
+
+GUI::gui_string AbbreviateWithTilde(const GUI::gui_string &path) {
+#if defined(GTK)
+	const GUI::gui_string_view homeDirectory = getenv("HOME");
+	if (StartsWith(path, homeDirectory)) {
+		return GUI::gui_string(GUI_TEXT("~")) + path.substr(homeDirectory.size());
+	}
+#endif
+	return path;
+}
+
+// Produce a menu or tab title from a buffer.
+// <index> <file name> <is read only> <is dirty>
+// 3 /src/example.cxx | *
+GUI::gui_string BufferTitle([[maybe_unused]] int pos, const Buffer &buffer, Title destination,
+	PropSetFile const &props, const Localization &localiser) {
+	GUI::gui_string title;
+
+	// Index
+#if defined(_WIN32) || defined(GTK)
+	if (pos < 10) {
+		const GUI::gui_string sPos = GUI::StringFromInteger((pos + 1) % 10);
+		const GUI::gui_string sHotKey = GUI_TEXT("&") + sPos + GUI_TEXT(" ");
+		if (destination == Title::menu) {
+			title = sHotKey;	// hotkey 1..0
+		} else {
+			if (props.GetInt("tabbar.hide.index") == 0) {
+#if defined(_WIN32)
+				title = sHotKey; // add hotkey to the tabbar
+#elif defined(GTK)
+				title = sPos + GUI_TEXT(" ");	// just the index
+#endif
+			}
+		}
+	}
+#endif
+
+	// File name or path
+	if (buffer.file.IsUntitled()) {
+		title += localiser.Text("Untitled");
+	} else {
+		if (destination == Title::menu) {
+			title += AbbreviateWithTilde(EscapeFilePath(buffer.file, destination));
+		} else {
+			title += EscapeFilePath(buffer.file.Name(), destination);
+		}
+	}
+
+	// Read only indicator
+	if (buffer.isReadOnly && props.GetInt("read.only.indicator")) {
+		title += GUI_TEXT(" |");
+	}
+
+	// Dirty indicator
+	if (buffer.isDirty) {
+		title += GUI_TEXT(" *");
+	}
+
+	return title;
+}
+
 }
 
 void SciTEBase::SetBuffersMenu() {
 	if (buffers.size() <= 1) {
 		DestroyMenuItem(menuBuffers, IDM_BUFFERSEP);
 	}
-	RemoveAllTabs();
+
+	std::vector<GUI::gui_string> tabNames;
 
 	int pos;
 	for (pos = buffers.lengthVisible; pos < bufferMax; pos++) {
@@ -1144,59 +1261,17 @@ void SciTEBase::SetBuffersMenu() {
 		SetMenuItem(menuBuffers, menuStart, IDM_BUFFERSEP, GUI_TEXT(""));
 		for (pos = 0; pos < buffers.lengthVisible; pos++) {
 			const int itemID = bufferCmdID + pos;
-			GUI::gui_string entry;
-			GUI::gui_string titleTab;
-
-#if defined(_WIN32) || defined(GTK)
-			if (pos < 10) {
-				GUI::gui_string sPos = GUI::StringFromInteger((pos + 1) % 10);
-				GUI::gui_string sHotKey = GUI_TEXT("&") + sPos + GUI_TEXT(" ");
-				entry = sHotKey;	// hotkey 1..0
-				if (props.GetInt("tabbar.hide.index") == 0) {
-#if defined(_WIN32)
-					titleTab = sHotKey; // add hotkey to the tabbar
-#elif defined(GTK)
-					titleTab = sPos + GUI_TEXT(" ");
-#endif
-				}
-			}
-#endif
-
-			if (buffers.buffers[pos].file.IsUntitled()) {
-				GUI::gui_string untitled = localiser.Text("Untitled");
-				entry += untitled;
-				titleTab += untitled;
-			} else {
-				GUI::gui_string path = buffers.buffers[pos].file.AsInternal();
-				GUI::gui_string filename = buffers.buffers[pos].file.Name().AsInternal();
-
-				EscapeFilePathsForMenu(path);
-#if defined(_WIN32)
-				// On Windows, '&' are also interpreted in tab names, so we need
-				// the escaped filename
-				EscapeFilePathsForMenu(filename);
-#endif
-				entry += path;
-				titleTab += filename;
-			}
-			// For short file names:
-			//char *cpDirEnd = strrchr(buffers.buffers[pos]->fileName, pathSepChar);
-			//strcat(entry, cpDirEnd + 1);
-
-			if (buffers.buffers[pos].isReadOnly && props.GetInt("read.only.indicator"))  {
-				entry += GUI_TEXT(" |");
-				titleTab += GUI_TEXT(" |");
-			}
-
-			if (buffers.buffers[pos].isDirty) {
-				entry += GUI_TEXT(" *");
-				titleTab += GUI_TEXT(" *");
-			}
-
+			const GUI::gui_string entry = BufferTitle(pos, buffers.buffers[pos],
+				Title::menu, props, localiser);
 			SetMenuItem(menuBuffers, menuStart + pos + 1, itemID, entry.c_str());
-			TabInsert(pos, titleTab.c_str());
+
+			const GUI::gui_string titleTab = BufferTitle(pos, buffers.buffers[pos],
+				Title::tab, props, localiser);
+			tabNames.push_back(titleTab);
 		}
 	}
+	UpdateTabs(tabNames);
+
 	CheckMenus();
 #if !defined(GTK)
 
@@ -1234,8 +1309,8 @@ void SciTEBase::SetFileStackMenu() {
 				sEntry = sHotKey;
 #endif
 
-				GUI::gui_string path = recentFileStack[stackPos].AsInternal();
-				EscapeFilePathsForMenu(path);
+				const GUI::gui_string path = EscapeFilePath(
+					recentFileStack[stackPos], Title::menu);
 
 				sEntry += path;
 				SetMenuItem(menuFile, MRU_START + stackPos + 1, itemID, sEntry.c_str());
@@ -1257,7 +1332,7 @@ bool SciTEBase::AddFileToBuffer(const BufferState &bufferState) {
 				buffers.buffers[iBuffer].file.selection = bufferState.file.selection;
 				buffers.buffers[iBuffer].foldState = bufferState.foldState;
 				buffers.buffers[iBuffer].bookmarks = bufferState.bookmarks;
-				if (buffers.buffers[iBuffer].lifeState == Buffer::opened) {
+				if (buffers.buffers[iBuffer].lifeState == Buffer::LifeState::opened) {
 					// File was opened synchronously
 					RestoreState(buffers.buffers[iBuffer], true);
 					DisplayAround(buffers.buffers[iBuffer].file);
@@ -1415,7 +1490,7 @@ bool SciTEBase::ToolIsImmediate(int item) {
 	std::string command = props.GetWild(propName.c_str(), FileNameExt().AsUTF8().c_str());
 	if (command.length()) {
 		JobMode jobMode(props, item, FileNameExt().AsUTF8().c_str());
-		return jobMode.jobType == jobImmediate;
+		return jobMode.jobType == JobSubsystem::immediate;
 	}
 	return false;
 }
@@ -1468,7 +1543,7 @@ void SciTEBase::SetToolsMenu() {
 
 JobSubsystem SciTEBase::SubsystemType(const char *cmd) {
 	std::string subsystem = props.GetNewExpandString(cmd, FileNameExt().AsUTF8().c_str());
-	return subsystem.empty() ? jobCLI : SubsystemFromChar(subsystem.at(0));
+	return subsystem.empty() ? JobSubsystem::cli : SubsystemFromChar(subsystem.at(0));
 }
 
 void SciTEBase::ToolsMenu(int item) {
@@ -1476,21 +1551,21 @@ void SciTEBase::ToolsMenu(int item) {
 
 	const std::string itemSuffix = StdStringFromInteger(item) + ".";
 	const std::string propName = std::string("command.") + itemSuffix;
-	std::string command(props.GetWild(propName.c_str(), FileNameExt().AsUTF8().c_str()).c_str());
+	std::string command(props.GetWild(propName.c_str(), FileNameExt().AsUTF8().c_str()));
 	if (command.length()) {
 		JobMode jobMode(props, item, FileNameExt().AsUTF8().c_str());
-		if (jobQueue.IsExecuting() && (jobMode.jobType != jobImmediate))
+		if (jobQueue.IsExecuting() && (jobMode.jobType != JobSubsystem::immediate))
 			// Busy running a tool and running a second can cause failures.
 			return;
-		if (jobMode.saveBefore == 2 || (jobMode.saveBefore == 1 && (!(CurrentBuffer()->isDirty) || Save())) || SaveIfUnsure() != saveCancelled) {
+		if (jobMode.saveBefore == 2 || (jobMode.saveBefore == 1 && (!(CurrentBuffer()->isDirty) || Save())) || SaveIfUnsure() != SaveResult::cancelled) {
 			if (jobMode.isFilter)
 				CurrentBuffer()->fileModTime -= 1;
-			if (jobMode.jobType == jobImmediate) {
+			if (jobMode.jobType == JobSubsystem::immediate) {
 				if (extender) {
 					extender->OnExecute(command.c_str());
 				}
 			} else {
-				AddCommand(command.c_str(), "", jobMode.jobType, jobMode.input, jobMode.flags);
+				AddCommand(command, "", jobMode.jobType, jobMode.input, jobMode.flags);
 				if (jobQueue.HasCommandToRun())
 					Execute();
 			}
@@ -1498,6 +1573,8 @@ void SciTEBase::ToolsMenu(int item) {
 	}
 }
 
+namespace {
+
 static SA::Line DecodeMessage(const char *cdoc, std::string &sourcePath, int format, SA::Position &column) {
 	sourcePath.clear();
 	column = -1; // default to not detected
@@ -1845,13 +1922,13 @@ static SA::Line DecodeMessage(const char *cdoc, std::string &sourcePath, int for
 	return -1;
 }
 
-#define CSI "\033["
+constexpr const char *CSI = "\033[";
 
-static bool SeqEnd(int ch) noexcept {
+constexpr bool SeqEnd(int ch) noexcept {
 	return (ch == 0) || ((ch >= '@') && (ch <= '~'));
 }
 
-static void RemoveEscSeq(std::string &s) {
+void RemoveEscSeq(std::string &s) {
 	size_t csi = s.find(CSI);
 	while (csi != std::string::npos) {
 		size_t endSeq = csi + 2;
@@ -1863,14 +1940,12 @@ static void RemoveEscSeq(std::string &s) {
 }
 
 // Remove up to and including ch
-static void Chomp(std::string &s, char ch) {
+void Chomp(std::string &s, char ch) {
 	const size_t posCh = s.find(ch);
 	if (posCh != std::string::npos)
 		s.erase(0, posCh + 1);
 }
 
-namespace {
-
 char Severity(const std::string &message) noexcept {
 	if (message.find("fatal") != std::string::npos)
 		return 3;
@@ -1894,7 +1969,7 @@ void SciTEBase::ShowMessages(SA::Line line) {
 	while ((line < maxLine) && (acc.StyleAt(acc.LineStart(line)) != SCE_ERR_CMD)) {
 		const SA::Position startPosLine = wOutput.LineStart(line);
 		const SA::Position lineEnd = wOutput.LineEnd(line);
-		std::string message = wOutput.StringOfRange(SA::Range(startPosLine, lineEnd));
+		std::string message = wOutput.StringOfSpan(SA::Span(startPosLine, lineEnd));
 		std::string source;
 		SA::Position column = 0;
 		int style = acc.StyleAt(startPosLine);
@@ -1967,7 +2042,7 @@ void SciTEBase::GoMessage(int dir) {
 					      "error.marker.back", ColourRGB(0xff, 0xff, 0)));
 			wOutput.MarkerAdd(lookLine, 0);
 			wOutput.SetSel(startPosLine, startPosLine);
-			std::string message = wOutput.StringOfRange(SA::Range(startPosLine, startPosLine + lineLength));
+			std::string message = wOutput.StringOfSpan(SA::Span(startPosLine, startPosLine + lineLength));
 			if ((style == SCE_ERR_ESCSEQ) || (style == SCE_ERR_ESCSEQ_UNKNOWN) || (style >= SCE_ERR_ES_BLACK)) {
 				// GCC message with ANSI escape sequences
 				RemoveEscSeq(message);
@@ -2057,7 +2132,7 @@ void SciTEBase::GoMessage(int dir) {
 					// Get the position in line according to current tab setting
 					startSourceLine = wEditor.FindColumn(sourceLine, column);
 				}
-				EnsureRangeVisible(wEditor, SA::Range(startSourceLine));
+				EnsureRangeVisible(wEditor, SA::Span(startSourceLine));
 				if (props.GetInt("error.select.line") == 1) {
 					//select whole source source line from column with error
 					SetSelection(endSourceline, startSourceLine);
