@@ -9,7 +9,7 @@
 #ifndef SPLITVECTOR_H
 #define SPLITVECTOR_H
 
-namespace Scintilla {
+namespace Scintilla::Internal {
 
 template <typename T>
 class SplitVector {
@@ -26,27 +26,34 @@ protected:
 	/// hence be fast.
 	void GapTo(ptrdiff_t position) noexcept {
 		if (position != part1Length) {
-			if (position < part1Length) {
-				// Moving the gap towards start so moving elements towards end
-				std::move_backward(
-					body.data() + position,
-					body.data() + part1Length,
-					body.data() + gapLength + part1Length);
-			} else {	// position > part1Length
-				// Moving the gap towards end so moving elements towards start
-				std::move(
-					body.data() + part1Length + gapLength,
-					body.data() + gapLength + position,
-					body.data() + part1Length);
+			try {
+				if (gapLength > 0) {	// If gap to move
+					// This can never fail but std::move and std::move_backward are not noexcept.
+					if (position < part1Length) {
+						// Moving the gap towards start so moving elements towards end
+						std::move_backward(
+							body.data() + position,
+							body.data() + part1Length,
+							body.data() + gapLength + part1Length);
+					} else {	// position > part1Length
+						// Moving the gap towards end so moving elements towards start
+						std::move(
+							body.data() + part1Length + gapLength,
+							body.data() + gapLength + position,
+							body.data() + part1Length);
+					}
+				}
+				part1Length = position;
+			} catch (...) {
+				// Ignore any exception
 			}
-			part1Length = position;
 		}
 	}
 
 	/// Check that there is room in the buffer for an insertion,
 	/// reallocating if more space needed.
 	void RoomFor(ptrdiff_t insertionLength) {
-		if (gapLength <= insertionLength) {
+		if (gapLength < insertionLength) {
 			while (growSize < static_cast<ptrdiff_t>(body.size() / 6))
 				growSize *= 2;
 			ReAllocate(body.size() + insertionLength + growSize);
@@ -206,11 +213,12 @@ public:
 	/// Add some new empty elements.
 	/// InsertValue is good for value objects but not for unique_ptr objects
 	/// since they can only be moved from once.
-	void InsertEmpty(ptrdiff_t position, ptrdiff_t insertLength) {
+	/// Callers can write to the returned pointer to transform inputs without copies.
+	T *InsertEmpty(ptrdiff_t position, ptrdiff_t insertLength) {
 		PLATFORM_ASSERT((position >= 0) && (position <= lengthBody));
 		if (insertLength > 0) {
 			if ((position < 0) || (position > lengthBody)) {
-				return;
+				return nullptr;
 			}
 			RoomFor(insertLength);
 			GapTo(position);
@@ -222,6 +230,7 @@ public:
 			part1Length += insertLength;
 			gapLength -= insertLength;
 		}
+		return body.data() + position;
 	}
 
 	/// Ensure at least length elements allocated,
@@ -278,7 +287,7 @@ public:
 	}
 
 	/// Retrieve a range of elements into an array
-	void GetRange(T *buffer, ptrdiff_t position, ptrdiff_t retrieveLength) const noexcept {
+	void GetRange(T *buffer, ptrdiff_t position, ptrdiff_t retrieveLength) const {
 		// Split into up to 2 ranges, before and after the split then use memcpy on each.
 		ptrdiff_t range1Length = 0;
 		if (position < part1Length) {
@@ -321,6 +330,16 @@ public:
 		}
 	}
 
+	/// Return a pointer to a single element.
+	/// Does not rearrange the buffer.
+	const T *ElementPointer(ptrdiff_t position) const noexcept {
+		if (position < part1Length) {
+			return body.data() + position;
+		} else {
+			return body.data() + position + gapLength;
+		}
+	}
+
 	/// Return the position of the gap within the buffer.
 	ptrdiff_t GapPosition() const noexcept {
 		return part1Length;
