@@ -105,10 +105,9 @@ std::string ComboSelectionText(GUI::Window w) {
 	const int selection = ComboBox_GetCurSel(combo);
 	if (selection != CB_ERR) {
 		const int len = ComboBox_GetLBTextLen(combo, selection);
-		GUI::gui_string itemText(len+1, L'\0');
-		const int lenActual = ComboBox_GetLBText(combo, selection, &itemText[0]);
+		GUI::gui_string itemText(len, L'\0');
+		const int lenActual = ComboBox_GetLBText(combo, selection, itemText.data());
 		if (lenActual != CB_ERR) {
-			itemText.pop_back(); // Remove NUL
 			return GUI::UTF8FromString(itemText);
 		}
 	}
@@ -1062,9 +1061,9 @@ LRESULT FindReplaceStrip::EditColour(HWND hwnd, HDC hdc) noexcept {
 
 void FindReplaceStrip::SetFindFromSource(ChangingSource source) {
 	if (source == ChangingSource::edit) {
-		pSearcher->SetFindText(ControlText(wText).c_str());
+		pSearcher->SetFindText(ControlText(wText));
 	} else {
-		pSearcher->SetFindText(ComboSelectionText(wText).c_str());
+		pSearcher->SetFindText(ComboSelectionText(wText));
 	}
 }
 
@@ -1591,7 +1590,6 @@ void FilterStrip::Size() {
 	GUI::Rectangle rcArea = LineArea(0);
 	rcArea.left += space;
 
-	const Interval verticalButton(rcArea.top - 1, rcArea.bottom);
 	const Interval verticalCheck(rcArea.top, rcArea.bottom);
 
 	const int checkWidth = rcArea.Height() - 1;	// Using height to make square
